<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentFreekassa_settings_mapper
{
    private $createOrderUrl = 'https://pay.freekassa.ru/';

    public function success()
    {
        if (
            !isset($_REQUEST['MERCHANT_ID']) ||
            !isset($_REQUEST['AMOUNT']) ||
            !isset($_REQUEST['MERCHANT_ORDER_ID']) ||
            !isset($_REQUEST['SIGN'])
        )
        {
        	header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
            print 'bad';
            return;
        }

        $orderId = (int) $_REQUEST['MERCHANT_ORDER_ID'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print 'YES';
            return;
        }

        if ($ord2->payment()->settings('kassaId') != $_REQUEST['MERCHANT_ID'])
        {
            throw new NotFoundException('bad');
        }

        if (
            !$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $ord2->price() != $_REQUEST['AMOUNT']
           )
        {
            header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());

        if (md5(
                $ord2->payment()->settings('kassaId').':'.
                $ord2->price().':'.
                $ord2->payment()->settings('secret2').':'.
                $ord2->id()
            ) == $_REQUEST['SIGN'])
        {
            $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
            $orderEvent->run('paid');

            print 'YES';
            return;
        }

        header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
        throw new NotFoundException('bad');
    }

    public function getPaymentUrl(Order $ord2)
    {
        $data = [
            'm' => $ord2->payment()->settings('kassaId'),
            'oa' => $ord2->price(),
            'o' => $ord2->id(),
            'currency' => $ord2->payment()->currency(),
            's' => md5(
                $ord2->payment()->settings('kassaId').':'.
                $ord2->price().':'.
                $ord2->payment()->settings('secret1').':'.
                $ord2->payment()->currency().':'.
                $ord2->id()
            )
        ];

        if (CUR_LANG == 'ru' || CUR_LANG == 'uk' || CUR_LANG == 'kk')
        {
            $data['lang'] = 'ru';
        }

        return $this->createOrderUrl.'?'.http_build_query($data);
    }

    public function settings($p)
    {
        if (!isset($p['settings']['kassaId']))
        {
            $p['settings']['kassaId'] = '';
        }

        if (!isset($p['settings']['secret1']))
        {
            $p['settings']['secret1'] = '';
        }

        if (!isset($p['settings']['secret2']))
        {
            $p['settings']['secret2'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}