<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentInterkassa_settings_mapper
{
    private $createOrderUrl = 'https://sci.interkassa.com/';

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

        if (!isset($_REQUEST['ik_inv_st']) ||
            !isset($_REQUEST['ik_co_id']) ||
            !isset($_REQUEST['ik_pm_no']) ||
            !isset($_REQUEST['ik_am']) ||
            !isset($_REQUEST['ik_sign']))
        {
        	header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
            print 'bad';
            return;
        }

        if ($_REQUEST['ik_inv_st'] !== 'success' || $_REQUEST['ik_am'] <= 0)
        {
            header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
            throw new NotFoundException('bad');
        }

        $orderId = (int) $_REQUEST['ik_pm_no'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print 'OK';
            return;
        }

        if ($ord2->payment()->settings('kassaId') != $_REQUEST['ik_co_id'])
        {
            throw new NotFoundException('bad');
        }

        if (
            !$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $ord2->price() != $_REQUEST['ik_am'] ||
            ($ord2->payment()->currency() != '' && $_REQUEST['ik_cur'] != $ord2->payment()->currency())
           )
        {
            header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());

		if (isset($_POST['ik_inv_st'])) $dataSet = $_POST;
		else $dataSet = $_GET;
		unset($dataSet['ik_sign']);
		ksort($dataSet, SORT_STRING);
		if ($_REQUEST['ik_pw_via'] == 'test_interkassa_test_xts')
        {
            array_push($dataSet, $ord2->payment()->settings('test'));
        }
		else
        {
            array_push($dataSet, $ord2->payment()->settings('secretKey'));
        }

        if (base64_encode(md5(implode(':', $dataSet), true)) == $_REQUEST['ik_sign'])
        {
            $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
            $orderEvent->run('paid');

            print 'OK';
            return;
        }

        header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
        throw new NotFoundException('bad');
    }

    public function getPaymentUrl(Order $ord2)
    {
        $dataSet = [];
        $dataSet['ik_co_id'] = $ord2->payment()->settings('kassaId');
        if ($ord2->payment()->currency() != '') {
        	$dataSet['ik_cur'] = $ord2->payment()->currency();
        }
        $dataSet['ik_am'] = $ord2->price();
        $dataSet['ik_pm_no'] = $ord2->id();
        $dataSet['ik_desc'] = $ord2->name();
        if ($ord2->customer()->email()) {
            $IDN = new \idna_convert(array('idn_version'=>2008));
        	$dataSet['ik_cli'] = $IDN->encode($ord2->customer()->email());
        }

        ksort($dataSet, SORT_STRING);
        array_push($dataSet, $ord2->payment()->settings('secretKey'));
        $sign = base64_encode(md5(implode(':', $dataSet), true));
        array_pop($dataSet);

        return $this->createOrderUrl.'?'.http_build_query($dataSet).'&ik_sign='.urlencode($sign);
    }

    public function settings($p)
    {
        if (!isset($p['settings']['kassaId']))
        {
            $p['settings']['kassaId'] = '';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['test']))
        {
            $p['settings']['test'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}