<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentLiqpay_settings_mapper
{
    private $createOrderUrl = 'https://www.liqpay.ua/api/pay';

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

    	$success =
            isset($_POST['amount']) &&
            isset($_POST['currency']) &&
            isset($_POST['public_key']) &&
            isset($_POST['description']) &&
            isset($_POST['order_id']) &&
            isset($_POST['type']) &&
            isset($_POST['status']) &&
            isset($_POST['transaction_id']) &&
            isset($_POST['sender_phone']);

        if (!$success) {
            throw new NotFoundException('bad');
        }

        $orderId = (int) $_POST['order_id'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print 'OK';
            return;
        }

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $ord2->price() != $_REQUEST['amount'])
        {
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());

        if (base64_encode(sha1($ord2->payment()->settings('privateKey').
                               $_POST['amount'].
                               $_POST['currency'].
                               $_POST['public_key'].
                               $_POST['order_id'].
                               $_POST['type'].
                               $_POST['description'].
                               $_POST['status'].
                               $_POST['transaction_id'].
                               $_POST['sender_phone'], 1)) == $_POST['signature'])
        {
            if ($_POST['status'] == 'success' || $_POST['status'] == 'sandbox')
            {
                $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
                $orderEvent->run('paid');

                print 'OK';
                return;

            }
            elseif ($_POST['status'] == 'failure')
            {
                throw new NotFoundException('Cancel');
            }
        }

        throw new NotFoundException('bad');
    }

    public function getPaymentUrl(Order $ord2)
    {
        return $this->createOrderUrl.
               '?public_key='.$ord2->payment()->settings('publicKey').
               '&amount='.$ord2->price().
               '&currency='.$ord2->payment()->currency().
               '&order_id='.$ord2->id().
               '&description='.urlencode($ord2->name()).
               ($ord2->payment()->status() == 'test' ? '&sandbox=1' : '').
               '&language='.(CUR_LANG == 'ru' || CUR_LANG == 'uk' || CUR_LANG == 'kk' ? 'ru' : 'en').
               '&server_url='.urlencode(siteProtocol.'://'.DOMAIN.BASE_DIR.'/payment/Liqpay/callback.php').
               '&signature='.$this->cnb_signature($ord2).
               '&Encoding=utf-8';

    }

    private function cnb_signature(Order $ord2) {

        $result_url = '';
        $server_url = siteProtocol.'://'.DOMAIN.BASE_DIR.'/payment/Liqpay/callback.php';

        return base64_encode(sha1(
            $ord2->payment()->settings('privateKey').
            $ord2->price().
            $ord2->payment()->currency().
            $ord2->payment()->settings('publicKey').
            $ord2->id().
            $ord2->name().
            $result_url.
            $server_url
        ,1));

    }

    public function settings($p)
    {
        if (!isset($p['settings']['publicKey']))
        {
            $p['settings']['publicKey'] = '';
        }

        if (!isset($p['settings']['privateKey']))
        {
            $p['settings']['privateKey'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}