<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\ErrorMessageException;
use Shop\Exception\NotFoundException;

class paymentProdamus_settings_mapper
{
    private $createOrderUrl = 'https://demo.payform.ru/';

    public function success()
    {
        header('Content-Type: text/plain; charset=utf-8');

        $headers = apache_request_headers();

        if (empty($_POST))
        {
            http_response_code(400);
            throw new NotFoundException('request is empty');
        }
        else if (!isset($_POST['order_num']) || !isset($_POST['sum']) || !isset($_POST['payment_status']))
        {
            http_response_code(400);
            throw new NotFoundException('bad request');
        }
        else if ($_POST['payment_status'] != 'success')
        {
            http_response_code(400);
            throw new ErrorMessageException('payment status not success');
        }
        else if (empty($headers['Sign']))
        {
            http_response_code(400);
            throw new NotFoundException('signature not found');
        }

        $orderId = (int) $_POST['order_num'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            http_response_code(200);
            echo 'success';
            return;
        }

        if (!Hmac::verify($_POST, $ord2->payment()->settings('secretKey'), $headers['Sign']))
        {
            http_response_code(400);
            throw new ErrorMessageException('signature incorrect');
        }

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $ord2->price() != $_POST['sum'])
        {
            http_response_code(400);
            throw new ErrorMessageException('order incorrect');
        }

        includeLangConst($ord2->langId());

        $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
        $orderEvent->run('paid');

        http_response_code(200);
        echo 'success';
    }

    public function getPaymentUrl(Order $ord2)
    {
        $IDN = new \idna_convert(array('idn_version'=>2008));

        $userEmail = '';
		if ($ord2->customer()->email())
        {
            $userEmail = $IDN->encode($ord2->customer()->email());
        }

        $this->createOrderUrl = str_replace('demo.', $ord2->payment()->settings('domain').'.', $this->createOrderUrl);

        $quantity = (float) $ord2->productQuantity();

        $product = [
			'name' => $ord2->productName(),
			'price' => round($ord2->price() / $quantity, 2),
			'quantity' => $quantity,
		];

        if ($ord2->payment()->settings('sendCheck'))
        {
            $product['tax'] = ['tax_type' => $ord2->payment()->settings('tax')];
            if ($ord2->payment()->settings('paymentMethodType'))
            {
                $product['paymentMethod'] = $ord2->payment()->settings('paymentMethodType');
            }
            if ($ord2->payment()->settings('paymentSubjectType'))
            {
                $product['paymentObject'] = $ord2->payment()->settings('paymentSubjectType');
            }
        }

        $data = [
            'order_id' => $ord2->id(),
            'order_sum' => $ord2->price(),
            'customer_phone' => $ord2->customer()->phone(),
            'customer_email' => $userEmail,
            'customer_extra' => $ord2->name(),
            'payments_limit' => 1,
            'products' => [
                $product,
            ],
            'do' => 'pay',//link
            'sys' => '',
            'urlNotification' => siteProtocol.'://'.$IDN->decode(DOMAIN).BASE_DIR.'/payment/Prodamus/callback.php',
        ];

        if ($ord2->payment()->settings('successURL'))
        {
            $data['urlSuccess'] = $ord2->payment()->settings('successURL');
        }

        if ($ord2->payment()->status() == 'test')
        {
            $data['demo_mode'] = 1;
        }

        if ($ord2->payment()->currency())
        {
            $data['currency'] = $ord2->payment()->currency();
        }

        if ($ord2->payment()->settings('secretKey'))
        {
            $data['signature'] = Hmac::create($data, $ord2->payment()->settings('secretKey'));
        }

        return sprintf('%s?%s', $this->createOrderUrl, http_build_query($data));
    }

    public function settings($p)
    {
        if (!isset($p['settings']['domain']))
        {
            $p['settings']['domain'] = 'demo';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        if (!isset($p['settings']['tax']))
        {
            $p['settings']['tax'] = 0;
        }

        if (!isset($p['settings']['paymentSubjectType']))
        {
            $p['settings']['paymentSubjectType'] = '';
        }

        if (!isset($p['settings']['paymentMethodType']))
        {
            $p['settings']['paymentMethodType'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        $data['settings']['domain'] = trim(preg_replace("/^(www\.)/iUu", '', trim(preg_replace("/^(https?\:\/\/)/iUu", '', $data['settings']['domain']), '/')));
        return $data;
    }
}