<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentWebmoney_settings_mapper
{
    public function success()
    {
        if (empty($_POST) ||
            empty($_POST['LMI_PAYMENT_NO']) ||
            empty($_POST['LMI_PAYEE_PURSE']) ||
            (isset($_POST['LMI_PREREQUEST']) && $_POST['LMI_PREREQUEST'] == 1)
           )
        {
            throw new NotFoundException('YES');
        }

        $orderId = (int) $_POST['LMI_PAYMENT_NO'];

        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print 'YES';
            return;
        }

        $price = $ord2->price();
        $rate = (float) $ord2->payment()->settings('rate');
        if ($rate)
        {
            $price = $price / $rate;
        }
        $price = round($price, 2);

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $price <= 0 ||
            $_POST['LMI_PAYMENT_AMOUNT'] != $price)
        {
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());

		if ($_POST['LMI_PAYEE_PURSE'] != $ord2->payment()->settings('wallet'))
        {
            throw new NotFoundException('Incorrect '.walletID);
		}
        if (strtoupper($_POST['LMI_PAYEE_PURSE'][0]) != strtoupper($_POST['LMI_PAYER_PURSE'][0]))
        {
            throw new NotFoundException('Incorrect letter wallet');
        }
		if(strtoupper(hash('sha256', $_POST['LMI_PAYEE_PURSE'].$_POST['LMI_PAYMENT_AMOUNT'].$_POST['LMI_PAYMENT_NO'].$_POST['LMI_MODE'].$_POST['LMI_SYS_INVS_NO'].$_POST['LMI_SYS_TRANS_NO'].$_POST['LMI_SYS_TRANS_DATE'].$ord2->payment()->settings('secretKey').$_POST['LMI_PAYER_PURSE'].$_POST['LMI_PAYER_WM'])) !== $_POST['LMI_HASH'])
        {
            throw new NotFoundException('Incorrect HASH');
        }

        $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
        $orderEvent->run('paid');

        print 'YES';
    }

    public function getPaymentUrl(Order $ord2)
    {
        try
        {
            $price = $ord2->price();
            $rate = (float) $ord2->payment()->settings('rate');
            if ($rate)
            {
                $price = $price / $rate;
            }

            $price = round($price, 2);

            $postdata = '<merchant.request><signtags><wmid>'.xmlParse($ord2->payment()->settings('wmId')).'</wmid>
            <validityperiodinhours>72</validityperiodinhours>
            <sign></sign><sha256></sha256><md5></md5>
            <secret_key>'.xmlParse($ord2->payment()->settings('secretKey')).'</secret_key>
            </signtags><paymenttags>
            <lmi_payee_purse>'.xmlParse($ord2->payment()->settings('wallet')).'</lmi_payee_purse>
            <lmi_payment_amount>'.xmlParse($price).'</lmi_payment_amount>
            <lmi_payment_no>'.$ord2->id().'</lmi_payment_no>
            <lmi_payment_desc>'.xmlParse($ord2->name()).'</lmi_payment_desc>
            <lmi_onetimepay>1</lmi_onetimepay>
            </paymenttags></merchant.request>';

            $header = '';
            $header .= "POST /conf/xml/XMLTransSave.asp HTTP/1.1\r\n";
            $header .= "Host: merchant.webmoney.com\r\n";
            $header .= "Content-Type: application/x-www-form-urlencoded\r\n";
            $header .= "Content-Length: " . strlen($postdata) . "\r\n";
            $header .= "Connection: close\r\n\r\n";
            $response = '';

            $fp = fsockopen('ssl://merchant.webmoney.com', 443, $errno, $errstr, 30);
            if ($fp)
            {
                fputs($fp, $header.$postdata);
                while (!feof($fp)) {
                    $response .= fgets($fp, 1024);
                }
                @fclose($fp);
            }

            $hunks = explode("\r\n\r\n", trim($response), 2);
            $body = trim($hunks[count($hunks) - 1]);
            $retval = -100;
            $retdesc = $transtoken = '';
            $xmlres = @simplexml_load_string($body);

            $payurl = '';

            if ($xmlres)
            {
                $payurl = (string) $xmlres->payurl;

                $retval = (int)$xmlres->retval;
                $retdesc = htmlspecialchars_decode(strval($xmlres->retdesc));
                if ($retval !== 0)
                {
                    throw new NotFoundException('Webmoney server error'.($retdesc ? ': '.$retdesc.' #'.$retval : ''));
                }
            }

            return $payurl;

        }
        catch (\Throwable $e)
        {
            throw new NotFoundException($e->getMessage());
        }
    }

    public function settings($p)
    {
        if (!isset($p['settings']['wallet']))
        {
            $p['settings']['wallet'] = '';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['wmId']))
        {
            $p['settings']['wmId'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}