<?php
use Shop\Entity\Order;
use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Usecase\RunEventAndSaveOrderUsecase;
use Shop\Exception\NotFoundException;

class paymentYoomoney_settings_mapper {
    private $createOrderUrl = 'https://yoomoney.ru/quickpay/confirm.xml';
    public $paymentType = array('PC'=>'Yoomoney', 'AC'=>'VISA/MasterCard/Maestro', 'MC'=>'Mobile');

    public function success()
    {
        header('Content-Type: text/html; charset=utf-8');

        if (!isset($_POST['notification_type']) ||
            !isset($_POST['operation_id']) ||
            !isset($_POST['amount']) ||
            !isset($_POST['label']) ||
            !isset($_POST['sha1_hash']))
        {
        	header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
            throw new NotFoundException('bad');
        }

        $orderId = (int) $_POST['label'];
        $ordUsecase = new GetOrderByIdUsecase();
        $ord2 = $ordUsecase->run($orderId);

        if ($ord2->event() !== 'submit' && $ord2->event() !== 'work' && $ord2->event() !== 'cancel')
        {
            print 'OK';
            return;
        }

        if (!$ord2->payment()->id() ||
            $ord2->payment()->status() == 'off' ||
            $ord2->price() != $_POST['withdraw_amount'])
        {
            header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
            throw new NotFoundException('bad');
        }

        includeLangConst($ord2->langId());

        if (sha1($_POST['notification_type'].'&'.
                 $_POST['operation_id'].'&'.
                 $_POST['amount'].'&'.
                 $_POST['currency'].'&'.
                 $_POST['datetime'].'&'.
                 $_POST['sender'].'&'.
                 $_POST['codepro'].'&'.
                 $ord2->payment()->settings('secretKey').'&'.
                 $_POST['label']) == $_POST['sha1_hash'])
        {
            $orderEvent = new RunEventAndSaveOrderUsecase($ord2);
            $orderEvent->run('paid');

            print 'OK';
            return;
        }

        header($_SERVER['SERVER_PROTOCOL'].' 400 Bad Request', true, 400);
        throw new NotFoundException('bad');
    }

    public function getPaymentUrl(Order $ord2)
    {
        return $this->createOrderUrl.
               '?receiver='.$ord2->payment()->settings('wallet').
               '&sum='.$ord2->price().
               '&targets='.urlencode('#'.$ord2->id()).
               '&label='.$ord2->id().
               '&formcomment='.urlencode($ord2->name()).
               '&short-dest='.urlencode($ord2->name()).
               ($ord2->payment()->settings('successURL') ? '&successURL='.urlencode($ord2->payment()->settings('successURL')) : '').
               '&quickpay-form=shop&paymentType='.urlencode($ord2->payment()->method());

    }

    public function settings($p)
    {
        if (!isset($p['settings']['wallet']))
        {
            $p['settings']['wallet'] = '41000000000';
        }

        if (!isset($p['settings']['secretKey']))
        {
            $p['settings']['secretKey'] = '';
        }

        if (!isset($p['settings']['successURL']))
        {
            $p['settings']['successURL'] = '';
        }

        return $p;
    }

    public function update($data)
    {
        return $data;
    }
}