<?php
namespace Admin\Controller;

use Admin\Controller\Dto\FormHandlerSearchDto;
use Admin\Repository\SqlHandlerRepository;
use Admin\Usecase\DeleteHandlerUsecase;
use Admin\Usecase\UpdateHandlerUsecase;
use main;

class HandlerController
{
    private $handlerRepo;

    public function __construct()
    {
        $this->handlerRepo = new SqlHandlerRepository();
    }

    public function lists() {

        if (isset(main::$user['admins']['emailv'])) {

            $dto = FormHandlerSearchDto::fromRequest();

            $dataLang = main::mapper('lang_settings')->getAll();
            $langIds = main::mapper('page_core')->getLangs();

            $dmList = main::mapper('domain_core')->findList();
            $goals = array('' => '', 'order' => goal_order, 'appointment' => goal_appointment, 'contact' => goal_contact, 'callback' => goal_callback, 'payment' => goal_payment, 'subscription' => goal_subscription, 'requests' => '['.goal_order . '...' . goal_subscription.']', 'login' => goal_login, 'forgot' => goal_forgot);
            $hTypes = $this->handlerRepo->getHendlersTypes();
            $countLangs = count($langIds);

            $clist = $this->handlerRepo->getList($dto, $dataLang, $dmList, $goals, $countLangs);

            return renderTpl('core/admin/listHandlers.php',
                compact('dto', 'goals', 'dmList', 'dataLang', 'langIds', 'hTypes', 'clist')
            );

        }

    }

    public function update()
    {
        if (!isset(main::$user['admins']['emailv']))
        {
            return;
        }

        if (!empty($_REQUEST['isSave']) && isSubmit())
        {
            if (!file_exists(FILES_S.'handler')) mkdir(FILES_S.'handler');
            main::rule('isWrite_core', 'isWrite', FILES_S.'handler/');

            if (isset($_REQUEST['discount']))
            {
                $_REQUEST['discount'] = floatval(str_replace(',','.', preg_replace('/\s{1,}/u', '', $_REQUEST['discount'])));
            }

            if (isset($_REQUEST['settings']['number']))
            {
                foreach($_REQUEST['settings']['number'] as $key => $value)
                {
                    $_REQUEST['settings']['number'][$key] = floatval(str_replace(',','.', preg_replace('/\s{1,}/u', '', $value)));
                    $_REQUEST['settings']['sale'][$key] = (int) $_REQUEST['settings']['sale'][$key];
                }
            }

            if ($_REQUEST['handler'] == 'Telegram' && telegramToken == '')
            {
                main::rule('require_core', 'telegramToken', array('Telegram Bot Token', telegramToken));
                main::addError('<a href="'.ADMIN_DIR.'?action=settings&service=telegram_settings" style="color:#CC3300">'.integrations_settings.' > Telegram</a>');
            }

            if(!main::getErrors())
            {
                if (isset(main::$user['admins']['emaile']))
                {
                    $usecase = new UpdateHandlerUsecase();
                    $usecase->run($_REQUEST);
                }

                redirect('?service=handler_core&action=lists');
			}
		}
        else if (isset($_REQUEST['error']) && isset($_SESSION['errorPhpHand']) && isset($_REQUEST['handler']) &&  $_REQUEST['handler'] == 'Php')
        {
            main::addError('PHP error '.$_SESSION['errorPhpHand']);
        }

        $hTypes = $this->handlerRepo->getHendlersTypes();
        if (!isset($_REQUEST['handler']))
        {
            return renderTpl('core/admin/createHandler.php', compact('hTypes'));
        }
        else
        {
            $data = $this->handlerRepo->findMultiById($_REQUEST['id']);
            $dmList = main::mapper('domain_core')->findList();

            $dataLang = main::mapper('lang_settings')->getAll();
            $langIds = main::mapper('page_core')->getLangs();

            if (!isset($data['event'])) $data['event'] = 'submit';

            $events = $this->handlerRepo->events();
            $events['delete'] = [];

            return renderTpl('core/admin/editHandler.php', compact('hTypes', 'data', 'events', 'dmList', 'dataLang', 'langIds'));
        }

    }

	public function delete()
    {
		if ((!isset(main::$user['admins']['emaild'])) || empty($_REQUEST['id'])) {
            return;
        }

        $ids = $_REQUEST['id'];
        if (is_array($ids))
        {
            foreach($ids as $key => $value)
            {
                $ids[$key] = (int) $value;
            }
        }
        if (!is_array($ids))
        {
            $ids = [(int) $ids];
        }

        $usecase = new DeleteHandlerUsecase();
        $usecase->run($ids);

        redirect('?service=handler_core&action=lists');
    }

    public function sort()
    {
		if ((!isset(main::$user['admins']['emaile'])) || empty($_REQUEST['list']))
        {
            return;
        }

        $event = false;
        if (isset($_REQUEST['event']))
        {
            $event = $_REQUEST['event'];
        }
		$this->handlerRepo->sort($_REQUEST['list'], $event);
	}
}