<?php
namespace Admin\Controller;

use Admin\Controller\Dto\FormOrderSearchDto;
use Admin\Repository\SqlOrderRepository;
use Admin\Repository\SqlHandlerRepository;
use Shop\Usecase\ChangeOrderEventUsecase;
use Shop\Usecase\RunHandlerEventUsecase;

use main;

class OrderController
{
    private $orderRepo;

    public function __construct()
    {
        $this->orderRepo = new SqlOrderRepository();
    }

    public function lists() {

        if (!isset(main::$user['admins']['orderv'])) {
            return;
        }

        $dto = FormOrderSearchDto::fromRequest();

        $items = $this->orderRepo->getList($dto, 80);
        $countItems = $this->orderRepo->countItems($dto);

        $handlerRepo = new SqlHandlerRepository();
        $events = $handlerRepo->events();
        if (isset(main::$user['admins']['orderd'])) {
            $events['delete'] = [];
        }

        $dmList = main::mapper('domain_core')->findList();

        $dataLang = main::mapper('lang_settings')->getAll();
        $langIds = main::mapper('page_core')->getLangs();

        return renderTpl('core/admin/listOrders.php',
            compact('dto', 'events', 'items', 'countItems', 'dmList', 'dataLang', 'langIds')
        );

    }

    public function checkNew()
    {
        if (!isset(main::$user['admins']['orderv']))
        {
            return '0';
        }
        return $this->orderRepo->countActive();
    }

    public function orderEvent()
    {
        if ((!isset(main::$user['admins']['orderd']) && $_REQUEST['event'] == 'delete') ||
    	    (!isset(main::$user['admins']['ordere']) && $_REQUEST['event'] != 'delete')) {
                main::addError(Permissions1);
                return false;
        }

        $orderId = 0;
    	if (isset($_REQUEST['orderId'])) {
            $orderId = (int) $_REQUEST['orderId'];
        }

        try
        {
            $changeUsecase = new ChangeOrderEventUsecase();
            $changeUsecase->run($orderId, $_REQUEST['event']);
        }
        catch (\Throwable $e) {}
    }

    public function runHandler()
    {
        if (!isset(main::$user['admins']['ordere']))
        {
            main::addError(Permissions1);
            return;
        }

        $orderId = 0;
    	if (isset($_REQUEST['orderId']))
        {
            $orderId = (int) $_REQUEST['orderId'];
        }

        $handlerId = 0;
    	if (isset($_REQUEST['handlerId']))
        {
            $handlerId = (int) $_REQUEST['handlerId'];
        }

        if (!$orderId || !$handlerId)
        {
            main::addError('Empty orderId or handlerId');
            return;
        }

        try
        {
            $changeUsecase = new RunHandlerEventUsecase();
            $item = $changeUsecase->run($orderId, $handlerId);
            $this->renderHtmlHandler($orderId, $item);
        }
        catch (\Throwable $e) {}
    }
    private function renderHtmlHandler($orderId, $item)
    {
        if (!$item) return;

        if (!empty($item['baseName']))
        {
            print '<b>'.$item['baseName'].'</b>: ';
        }

        if (isset($item['error']))
        {
            print '<span style="color:red;">'.$item['value'].'</span>';
        }
        else
        {
            print $item['value'];
        }

        if (isset($item['handler']))
        {
            print '<span class="cursor" title="'.again.'" style="margin-left:10px;font-size:14px;color:#07367B;" onclick="runHandler('.$orderId.', '.$item['handler'].')">&#8634;</span>';
        }
    }

    public function multyChange()
    {
        $dto = FormOrderSearchDto::fromRequest();
		if (empty($_REQUEST['ids']))
        {
            redirect(ADMIN_DIR.'?service=order_core&action=lists&'.http_build_query($dto));
        }

        if ($_REQUEST['multyevent'] == 'delete')
        {
            if (!isset(main::$user['admins']['orderd']))
            {
                main::addError(Permissions1);
                redirect(ADMIN_DIR.'?service=order_core&action=lists&'.http_build_query($dto));
            }
        }
        else if (!isset(main::$user['admins']['ordere']))
        {
            main::addError(Permissions1);
            redirect(ADMIN_DIR.'?service=order_core&action=lists&'.http_build_query($dto));
        }

        $changeUsecase = new ChangeOrderEventUsecase();
        foreach($_REQUEST['ids'] as $orderId)
        {
            try
            {
                $orderId = (int) $orderId;
                $changeUsecase->run($orderId, $_REQUEST['multyevent']);
            }
            catch (\Throwable $e) {}
        }

        redirect(ADMIN_DIR.'?service=order_core&action=lists&'.http_build_query($dto));
    }

    public function export_form()
    {
		if (!isset(main::$user['admins']['orderv'])) return;

        $dto = FormOrderSearchDto::fromRequest();
		return renderTpl('core/admin/listOrdersExportFrom.php', compact('dto'));
	}

    public function file()
    {
        if (!isset(main::$user['admins']['orderv'])) return;
        if (empty($_REQUEST['file']) || empty($_REQUEST['id'])) return;

        if (is_file(FILES_S.'orders/'.$_REQUEST['id'].'/'.$_REQUEST['file'].'.txt')) {
            $path = FILES_S.'orders/'.$_REQUEST['id'].'/'.$_REQUEST['file'].'.txt';
        } else if (is_file(FILES_S.'orders/'.$_REQUEST['id'].'/'.$_REQUEST['file'])) {
            $path = FILES_S.'orders/'.$_REQUEST['id'].'/'.$_REQUEST['file'];
        } else {
            return;
        }

        session_write_close();
        header("Accept-Ranges: bytes");
        header("Content-Transfer-Encoding: binary");
        header("Expires: 0");
        header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
        header('Content-Disposition: attachment; filename="'.$_REQUEST['file'].'"');
        header('Content-Type: application/octet-stream');
        header("Content-Length: " .filesize($path));
        header('Last-Modified: '.gmdate('D, d M Y H:i:s', time()).' GMT', true);
        readfile($path);
        exit;
    }

    public function export()
    {
        if (!isset(main::$user['admins']['orderv'])) return;

        $_REQUEST['fields'] = trim(trim(preg_replace('/\s+/u', ' ', str_replace(',', ' ', $_REQUEST['fields']))), '%');
        $fields = explode('% %', $_REQUEST['fields']);

        $dto = FormOrderSearchDto::fromRequest();

		$list = $this->orderRepo->getList($dto);

        $filename = FILES_S.'orders'.main::$user['id'].'.csv';
        $fp = fopen($filename, 'w');
        fputs($fp, chr(0xEF) . chr(0xBB) . chr(0xBF)); // BOM

        fputcsv($fp, $fields, ';', '"');

        foreach($list as $order)
        {
            $vars = [
                '%paymentMethod%' => '',
                '%payStatus%' => paymentWaitPaid,
                '%orderPage%'=>'',
                '%utm%'=>'',
                '%utm_source%' => '',
                '%utm_medium%' => '',
                '%utm_campaign%' => '',
                '%utm_content%' => '',
                '%utm_term%' => '',
                '%reftail%' => '',
                '%roistat_visit%' => '',
                '%IP%' => '',
                '%productCodes%' => '',
                '%productSKU%' => ''
            ];
	        $vars['%orderId%'] = (int) $order['id'];
	        $vars['%orderName%'] = $order['orderName'];
	        $vars['%price%'] = (float) $order['price'];
            $vars['%formId%'] = $order['formId'];
            $vars['%productId%'] = (int) $order['productId'];
            $vars['%productName%'] = $order['orderName'];
            $vars['%productPrice%'] = (float) $order['productPrice'];
            $vars['%productQuantity%'] = (float) $order['quantity'];

            foreach($order['orderInfo'] as $key => $item) {
        		$vars['%'.$key.'%'] = $item['value'];
	        }

            if (!empty($vars['%productCodes%']))
            {
                $vars['%productCodes%'] = str_replace("\n", ",", $vars['%productCodes%']);
            }

            if (isset($order['orderInfo']['payStatus']) && !isset($order['orderInfo']['paymentMethod'])) {
                $vars['%paymentMethod%'] = $order['orderInfo']['payStatus']['baseName'];
            }

            $vars['%userEmail%'] = $order['userEmail'];
            $vars['%userPhone%'] = $order['userPhone'];
            $vars['%dateTime%'] = date('j F Y H:i:s', $order['dateCreate']);
            $vars['%date%'] = date('j F Y', $order['dateCreate']);

            $utms = false;
			if (!empty($vars['%utm%'])) $utms = $vars['%utm%'];
            if ($utms) {
			    $utms = explode("; ", $utms);
			    foreach($utms as $value) {
			       if (empty($value)) continue;
			       $value = explode(" = ", $value);
			       if (!empty($value[1]) && !empty($value[0])) {
			         $vars['%utm_'.$value[0]."%"] = $value[1];
			       }
			    }
            }

            foreach($fields as $index => $field) {
                if (isset($vars['%'.$field.'%'])) {
                    $values[$index] = strip_tags($vars['%'.$field.'%']);
                } else {
                    $values[$index] = '';
                }
            }
            fputcsv($fp, $values, ';', '"');
        }

        fclose($fp);

        header("Accept-Ranges: bytes");
        header("Content-Transfer-Encoding: binary");
        header("Expires: 0");
        header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
        header('Content-Disposition: attachment; filename="orders.csv"');
        header('Content-Type: application/octet-stream');
        header("Content-Length: " .filesize($filename));
        header('Last-Modified: '.gmdate('D, d M Y H:i:s', time()).' GMT', true);
        readfile($filename);
        unlink($filename);
        exit;
	}
}