<?php
namespace Shop\Controller;

use Shop\Usecase\GetOrderByIdUsecase;
use Shop\Repository\SqlOrderRepository;
use Shop\Exception\NotFoundException;
use Shop\Exception\ErrorMessageException;
use Common\Lib\DB;
use main;

class PaymentController
{
    public function __construct()
    {
        session_start();

        if (substr($_SERVER['HTTP_HOST'], 0, 4) === 'www.')
        {
            $_SERVER['HTTP_HOST'] = substr($_SERVER['HTTP_HOST'], 4);
        }
        $host = explode(':',$_SERVER['HTTP_HOST']);
        $_SERVER['HTTP_HOST'] = $host[0];
        define('DOMAIN', $host[0]);
    }

    public function pay()
    {
        login();

        try
        {
            $orderRep = new SqlOrderRepository();

            $orderId = isset($_REQUEST['orderId']) ? (int) $_REQUEST['orderId'] : 0;
            $paymentId = isset($_REQUEST['paymentId']) ? (int) $_REQUEST['paymentId'] : 0;

            if (!$paymentId)
            {
                throw new NotFoundException('Payment system error');
            }

            if (!$orderId)
            {
                throw new NotFoundException('Order id error');
            }

            $ordUsecase = new GetOrderByIdUsecase();
            $ord2 = $ordUsecase->run($orderId);
            if (!$ord2->price()) {
                throw new NotFoundException('Error');
            }

            includeLangConst($ord2->langId());

            if ($paymentId !== $ord2->payment()->id())
            {
                $payment = $orderRep->findPaymentById($paymentId, $ord2->payment()->event());
                $ord2->setPayment($payment);
                if ($ord2->payment()->status() == 'off')
                {
                    throw new NotFoundException('Payment system error');
                }
            }

            try
            {
                $link = main::mapper('payment'.$ord2->payment()->system().'_settings')->getPaymentUrl($ord2);
                header('location: '.$link);
            }
            catch (\Throwable $e)
            {
                $ord2->updateOrderInfo('paymentLink', [
                    'baseName' => 'Payment',
                    'value' => escape($e->getMessage()),
                    'error' => 1
                ]);
                print 'Payment error';
            }

            $orderRep->save($ord2);

        }
        catch (NotFoundException | ErrorMessageException $e)
        {
            print $e->getMessage();
        }

        DB::connection()->close();
    }

    public function callback($name)
    {
        try
        {
            main::mapper('payment'. $name .'_settings')->success();
        }
        catch (NotFoundException | ErrorMessageException $e)
        {
            print $e->getMessage();
        }

        DB::connection()->close();
    }

}