<?php
namespace Shop\Entity;

use Shop\Entity\Partner;

class Customer
{
    private $id = 0;
    private $langId = 3;
    private $name = '';
    private $email = '';
    private $phone = '';
    private $login = '';
    private $password = '';
    private $money = 0;
    private $promoCode = '';
    private $saleCount = 0;
    private $saleCountPlus = 0;
    private $couponCounterIncrease = ['Order' => 0, 'Paid' => 0];
    private $isPartner = 0;
    private $partner;
    private $ip = '';
    private $operation = ''; // 'insert', 'update'
    private $groups = [7];
    private $status = 1;
    private $timers = [];

    public function __construct($id, $langId, $name, $email, $phone, $login, $password, $money, $promoCode, $saleCount, $isPartner, Partner $partner, $groups, $ip, $status = 1)
    {
        $this->id = (int) $id;
        $this->langId = (int) $langId;
        $this->name = $name;
        $this->email = $email;
        $this->phone = $phone;
        $this->login = $login;
        $this->password = $password;
        $this->money = (float) $money;
        $this->promoCode = $promoCode;
        $this->saleCount = (int) $saleCount;
        $this->partner = $partner;
        $this->isPartner = (int) $isPartner;
        if ($id && $this->partner->id() && $id == $this->partner->id()) {
            $this->isPartner = 1;
        }
        if ($groups)
        {
            $this->groups = $groups;
        }
        $this->ip = $ip;
        $this->status = (int) $status;
        if ($this->id)
        {
            $this->operation = 'update';
        }
    }

    public function id()
    {
        return $this->id;
    }
    public function setId($id)
    {
        $this->id = (int) $id;
    }

    public function langId()
    {
        return $this->langId;
    }
    public function setLangId($langId)
    {
        $this->langId = (int) $langId;
    }

    public function name()
    {
        return $this->name;
    }
    public function setName($name)
    {
        $this->name = $name;
    }

    public function login()
    {
        if ($this->login)
        {
            return $this->login;
        }
        return $this->email ? $this->email : $this->phone;
    }

    public function email()
    {
        return $this->email;
    }
    public function setEmail($email)
    {
        $this->email = $email;
    }

    public function phone()
    {
        return $this->phone;
    }
    public function setPhone($phone)
    {
        $this->phone = $phone;
    }

    public function password()
    {
        if ($this->operation == 'insert' && !$this->password) {
            $this->password = substr(md5(rand(0, 99999)), rand(0, 4), rand(7, 12));
        }
        return $this->password;
    }

    public function promo1SaleCount()
    {
        return $this->saleCount;
    }
    public function saleCountPlus()
    {
        return $this->saleCountPlus;
    }
    public function addSaleCount($saleCount)
    {
        $saleCount = (int) $saleCount;
        $this->saleCount += $saleCount;
        $this->saleCountPlus += $saleCount;
    }

    public function couponCounterIncrease($event)
    {
        return $this->couponCounterIncrease[$event];
    }
    public function setCouponCounterIncrease($event)
    {
        $this->couponCounterIncrease[$event] = 1;
    }

    public function money()
    {
        return $this->money;
    }
    public function setMoney($money)
    {
        $this->money = (float) $money;
    }

    public function promoCode()
    {
        return $this->promoCode;
    }

    public function isPartner()
    {
        return $this->isPartner;
    }
    public function setIsPartner($isPartner)
    {
        $this->isPartner = (int) $isPartner;
    }

    public function partner()
    {
        return $this->partner;
    }

    public function ip()
    {
        return $this->ip;
    }

    public function addGroup($id)
    {
        if (!$id || in_array($id, $this->groups))
        {
            return;
        }

        $guest = array_search(7, $this->groups);
        if ($guest !== false)
        {
            unset($this->groups[$guest]);
        }

        $this->groups[] = (int) $id;
    }

    public function groups()
    {
        return $this->groups;
    }

    public function status()
    {
        return $this->status;
    }
    public function setStatus($status)
    {
        $this->status = (int) $status;
    }

    public function operation()
    {
        return $this->operation;
    }
    public function setOperation($operation = 'insert')
    {
        if ($this->id) {
            $operation = 'update';
        }
        $this->operation = $operation;
    }

    public function timer($id)
    {
        return isset($this->timers[$id]) ? $this->timers[$id] : 0;
    }
    public function setTimers($timers)
    {
        $this->timers = $timers;
    }

}
