<?php
namespace Shop\Repository;

use Shop\Entity\Order;
use Shop\Entity\Payment;
use Shop\Entity\Customer;
use Shop\Entity\Partner;
use Shop\Exception\NotFoundException;
use Exception;
use Common\Lib\DB;
use main;

class SqlOrderRepository
{
    private $con;
    private $pomoCounter = 1;

    public function __construct()
    {
        $this->con = DB::connection();
    }

    public function findOrderById($id)
    {
        if (!$id)
        {
            throw new NotFoundException('Not found');
        }

        $r = $this->con->query('SELECT *,
                                       UNIX_TIMESTAMP(dateCreate) as dateCreate,
                                       UNIX_TIMESTAMP(dateUpdate) as dateUpdate
                                  FROM '.DB_PREFIX.'order
                                 WHERE id = '. (int) $id.'
                                 LIMIT 1');
        $result = $r->fetch_assoc();
        if (!$result)
        {
            throw new NotFoundException('Not found');
        }

        $result['orderInfo'] = $result['orderInfo'] ? unserialize($result['orderInfo']) : [];
        return $result;

    }

    public function insert(Order $ord2)
    {
        $userEmail = $ord2->customer()->email();
        if (mb_strlen($userEmail, 'UTF-8') > 120) {
            $userEmail = mb_substr($userEmail, 0 , 120, 'UTF-8');
        }

        $userPhone = $ord2->customer()->phone();
        if (mb_strlen($userPhone, 'UTF-8') > 28) {
            $userPhone = mb_substr($userPhone, 0 , 28, 'UTF-8');
        }

    	$this->con->query('INSERT INTO '.DB_PREFIX.'order (domainId,
                                                           langId,
        												   price,
        												   orderName,
                                                           quantity,
                                                           productPrice,
                                                           increase,
        												   formId,
        												   goal,
        												   productId,
        												   pageId,
        												   userEmail,
        												   userPhone,
                                                           paymentId,
                                                           dateUpdate,
                                                           dateCreate,
                                                           orderInfo)
                                                   VALUES ('.$ord2->domainId().',
                                                           '.$ord2->langId().',
                                                   		   "'.$this->con->real_escape_string($ord2->price()).'",
                                                   		   "'.$this->con->real_escape_string($ord2->name()).'",
                                                   		   '.$ord2->productQuantity().',
                                                           '.$ord2->productPrice().',
                                                           '.$ord2->increase().',
                                                   		   '.$ord2->formId().',
                                                   		   "'.$ord2->goal().'",
                                                   		   '.$ord2->productId().',
                                                   		   '.$ord2->pageId().',
                                                           "'.$this->con->real_escape_string($userEmail).'",
                                                           "'.$this->con->real_escape_string($userPhone).'",
                                                           '.$ord2->payment()->id().',
                                                           FROM_UNIXTIME('.$ord2->dateUpdate().'),
                                                           FROM_UNIXTIME('.$ord2->dateCreate().'),
                                                   		   "'.(!empty($ord2->orderInfo()) ? $this->con->real_escape_string(serialize($ord2->orderInfo())) : '').'")');

        $ord2->setId($this->con->insert_id);
    }

    public function save(Order $ord2)
    {
        $customer = $ord2->customer();

        if ($customer->operation())
        {
            if ($customer->operation() === 'insert')
            {
                $customerId = $this->insertCustomer($customer);
                $customer->setId($customerId);
            }

            $customer->setIsPartner(
                main::mapper('group_ACL')->isPartnerGroup($customer->groups())
            );

            $this->updateCustomer($customer);
        }

        if ($ord2->operation() === 'delete')
        {
            $this->delete($ord2->id());
        }
        else if ($ord2->operation())
        {
            $this->update($ord2);
        }

        if ($customer->partner()->isSaveCommission())
        {
            $this->updatePartnerCommission($customer->partner());
        }

        if ($customer->couponCounterIncrease('Order'))
        {
            $this->couponCounterIncrease($ord2->couponId(), 'Order');
        }

        if ($customer->couponCounterIncrease('Paid'))
        {
            $this->couponCounterIncrease($ord2->couponId(), 'Paid');
        }

    }

    private function update(Order $ord2)
    {
        $this->con->query('UPDATE '.DB_PREFIX.'order
                             SET
                                price = "'.$this->con->real_escape_string($ord2->price()).'",
                                userId = '.$ord2->customer()->id().',
                                couponId = '.$ord2->couponId().',
                                partner1Id = '.$ord2->customer()->partner()->id().',
                                dateUpdate = FROM_UNIXTIME('.$ord2->dateUpdate().'),
                                paymentId = '. $ord2->payment()->id() .',
                                isSend = '. $ord2->isSend() .',
                                paymentStatus = "'.$this->con->real_escape_string($ord2->payment()->event()).'",
                                event = "'.$this->con->real_escape_string($ord2->event()).'",
                                orderInfo = "'.(!empty($ord2->orderInfo()) ? $this->con->real_escape_string(serialize($ord2->orderInfo())) : '').'"
                             WHERE id = "'.$ord2->id().'"');

    }

    private function delete($id)
    {
        if (file_exists(FILES_S.'orders/'.$id))
        {
            $files = selectDirFiles(FILES_S.'orders/'.$id);
            foreach($files as $file)
            {
                @unlink(FILES_S.'orders/'.$id.'/'.$file);
            }
            @rmdir(FILES_S.'orders/'.$id);
        }
        $this->con->query('DELETE FROM '.DB_PREFIX.'order WHERE id ='. (int) $id .' LIMIT 1');
    }

    public function findPaymentById($id, $event = 'Start')
    {
        $id = (int) $id;

    	if (empty($id))
        {
            return new Payment(0, 'Unknown', '', '', '', [], $event, 'off');
        }

        $r = $this->con->query('SELECT * FROM '.DB_PREFIX.'payment WHERE id = ' . $id);
        $data = $r->fetch_assoc();

        if (empty($data))
        {
            return new Payment(0, 'Unknown', '', '', '', [], $event, 'off');
        }

        $data['settings'] = !empty($data['settings']) ? unserialize($data['settings']) : [];

        return new Payment(
            $data['id'],
            $data['baseName'],
            $data['paymentSystem'],
            $data['method'],
            $data['currency'],
            $data['settings'],
            $event,
            $data['status']
        );
    }

    public function findCouponById($id)
    {
        if (!$id)
        {
            throw new NotFoundException('Not found');
        }

        $r = $this->con->query('SELECT *, UNIX_TIMESTAMP(expire) as expire
                    FROM '.DB_PREFIX.'order_coupon
                    WHERE id = '. (int) $id);

        $return = $r->fetch_assoc();
        if (!$return)
        {
            throw new NotFoundException('Not found coupon');
        }

        return $return;

    }

    public function findCouponByCode($code)
    {
        if (!$code)
        {
            throw new NotFoundException('Not found');
        }

        $r = $this->con->query('SELECT *, UNIX_TIMESTAMP(expire) as expire
                    FROM '.DB_PREFIX.'order_coupon
                    WHERE code = "'.trim($this->con->real_escape_string($code)).'"');

        $return = $r->fetch_assoc();
        if (!$return)
        {
            throw new NotFoundException('Not found coupon');
        }

        return $return;

    }

    private function couponCounterIncrease($couponId, $usedTimeType = 'Order')
    {
        $this->con->query('UPDATE '.DB_PREFIX.'order_coupon
                              SET usedTime'.$usedTimeType.'Count = usedTime'.$usedTimeType.'Count + 1
                            WHERE id = '. (int) $couponId);
    }

    public function getCustomerById($id)
    {
        $r = $this->con->query('SELECT * FROM '.DB_PREFIX.'acl_user WHERE id = '. (int) $id);
        $return = $r->fetch_assoc();
        if ($return)
        {
            $return['groups'] = $this->getCustomerGroups($id);
        }
        return $return;
    }

    public function getCustomerByEmailAndPhone($email, $phone = '') {
        $where = '';
        if ($email) {
            $where = 'userEmail = "'.$this->con->real_escape_string($email).'"';
        }
        if ($phone) {
            $where .= ($where ? ' || ' : '').'userPhone = "'.$this->con->real_escape_string($phone).'"';
        }
        $r = $this->con->query('SELECT * FROM '.DB_PREFIX.'acl_user WHERE '.$where);
        $return = $r->fetch_assoc();
        if ($return)
        {
            $return['groups'] = $this->getCustomerGroups($return['id']);
        }
        return $return;
    }

    private function getCustomerGroups($id)
    {
    	$return = [];

        $r = $this->con->query('SELECT groupId FROM '.DB_PREFIX.'acl_ug WHERE userId = ' . (int) $id);
        while ($result = $r->fetch_assoc())
        {
            $return[] = $result['groupId'];
        }

        return $return;
    }

    private function insertCustomer(Customer $customer)
    {
        $this->con->query('INSERT INTO '.DB_PREFIX.'acl_user
                                        (userName,
                                         userEmail,
                                         userPhone,
                                         promoCode,
                                         uPassword,
                                         hPassword,
                                         langId,
                                         admins)
                                 VALUES ("'.$this->con->real_escape_string($customer->name()).'",
                                         "'.$this->con->real_escape_string($customer->email()).'",
                                         "'.$this->con->real_escape_string($customer->phone()).'",
                                         "'.$this->generatePromo($customer->email(), $customer->phone()).'",
                                         "'.$customer->password().'",
                                         "'.md5($customer->id().'499'.$customer->password()).'",
                                         '.$customer->langId().',
                                         "")');

        return $this->con->insert_id;
    }

    private function updateCustomer(Customer $customer)
    {
        $partnerId = 0;
        if (!$customer->isPartner())
        {
            $partnerId = $customer->partner()->id();
        }

        $this->con->query('UPDATE '.DB_PREFIX.'acl_user
                             SET status = '.$customer->status().',
                                 isPartner = '.$customer->isPartner().',
                                 partnerId = '.$partnerId.',
                                 promo1SaleCount = promo1SaleCount + '.$customer->saleCountPlus().'
                             WHERE id = '.$customer->id());

        $this->updateCustomerGroups($customer);

        if ($customer->saleCountPlus())
        {
            $this->setSalesHistory($customer->id());
        }

    }

    private function updateCustomerGroups(Customer $customer)
    {
        $groups = $customer->groups();

        $where = '';
        if (!empty($groups))
        {
            $where = ' AND groupId NOT IN ('.implode(',',$groups).')';
            $insert = [];
            foreach($groups as $groupId)
            {
                $insert[] .= '('.$customer->id().','. $groupId.')';
            }
            if (!empty($insert))
            {
                $this->con->query('INSERT IGNORE INTO '.DB_PREFIX.'acl_ug (userId, groupId) VALUES '.implode(',',$insert));
            }
        }

        $this->con->query('DELETE FROM '.DB_PREFIX.'acl_ug WHERE userId = '.$customer->id().$where);

    }

    private function generatePromo($email, $phone)
    {
    	if ($this->pomoCounter<9) {
            $promo = $this->con->real_escape_string('pr'.substr(md5($email.$phone.time()), rand(0, 4), 6).substr(md5(rand(0, 99999).' '.time()), rand(0, 4), 6));
    	} else {
    		$promo = $this->con->real_escape_string('prc'.substr(md5(rand(0, 99999).' '.time()), rand(0, 4), 6).substr(md5($email.$phone.time()), rand(0, 4), 6));
    	}
    	$r = $this->con->query('SELECT 1 FROM '.DB_PREFIX.'acl_user WHERE promoCode = "'.$promo.'"');
        $data = $r->fetch_assoc();
        if (!$data) {
            return $promo;
        }
        $this->pomoCounter++;
        return $this->generatePromo($email, $phone);
    }

    public function getPartnerById($id)
    {
        $r = $this->con->query('SELECT * FROM '.DB_PREFIX.'acl_user WHERE status = 1 AND  id = '. (int) $id);
        return $r->fetch_assoc();
    }

    public function getProduct($id)
    {
        if (empty($id)) return false;

        $r = $this->con->query('SELECT * FROM '.DB_PREFIX.'product WHERE `productId` = '. (int) $id);
        return $r->fetch_assoc();
    }

    public function getCodes($productId, $quantity)
    {
        $return = [];

        $this->lock($productId);

        $r = $this->con->query('SELECT * FROM '.DB_PREFIX.'product WHERE `productId` = '. (int) $productId);
        $data = $r->fetch_assoc();
        if (empty($data['codes']))
        {
            throw new NotFoundException('Not found');
        }

        $codes = explode("\n", $data['codes']);
        for ($i = 0; $i < $quantity; $i++)
        {
            if (isset($codes[$i]))
            {
                $return[] = $codes[$i];
                unset($codes[$i]);
            }
        }

        if (count($return) != $quantity)
        {
            throw new NotFoundException('Not found');
        }

        $this->con->query('UPDATE '.DB_PREFIX.'product
            SET
                codes = "'. $this->con->real_escape_string(implode("\n", $codes)) .'"
            WHERE
                productId = '. (int) $productId);

        return $return;
    }

    private function updatePartnerCommission(Partner $partner)
    {
        $this->con->query('UPDATE '.DB_PREFIX.'acl_user
                              SET promo1SaleCount = promo1SaleCount + 1,
                                  money = money + '. $partner->commission() .'
                            WHERE id = '. $partner->id());

        $this->setSalesHistory($partner->id());
    }

    private function setSalesHistory($userId)
    {
        $month = mktime(0, 0, 0, date("m") + 1, 0, date("Y"));
        $year = mktime(0, 0, 0, 0, 0, date("Y") + 1);

        $this->con->query('INSERT INTO '.DB_PREFIX.'acl_user_history (userId, sales1Month, sales1Year, month, year) VALUES
            ('.(int)$userId.', 1, 1, '.$month.','.$year.')
            ON DUPLICATE KEY UPDATE
            hostsMonth = IF (month = '.$month.', hostsMonth, 0),
            hostsYear = IF (year = '.$year.', hostsYear, 0) ,
            sales1Month = IF (month = '.$month.', sales1Month + 1, 1),
            sales1Year = IF (year = '.$year.', sales1Year + 1, 1),
            month = '.$month.', year = '.$year);
    }

    private function lock($productId)
    {
        $r = $this->con->query('SELECT GET_LOCK("codes_'.$productId.'", 1) as locked');
        $item = $r->fetch_assoc();
        if ($item && $item['locked'])
        {
            return true;
        }

        throw new Exception('System error');
    }

    public function unlock($productId)
    {
        $this->con->query('SELECT RELEASE_LOCK("codes_'.$productId.'")');
	}
}