<?php
namespace Shop\Usecase\Handler;

use Shop\Entity\Order;
use Shop\Exception\NotFoundException;

class EmailUsecase
{
    private $ord2;
    private $handler;

    private $IDN;
	private $socket;
	private $smpt_response;

    public function __construct(Order $ord2, $handler = [])
    {
        $this->ord2 = $ord2;
        $this->handler = $handler;
        $this->IDN = new \idna_convert(array('idn_version'=>2008));
    }

    public function run()
    {

        $vars = $this->ord2->vars();
        $seaMd = array_keys($vars);
        $repMd = array_values($vars);

        $data = [
            'emailTo' => '',
            'replyTo' => '',
            'emailFrom' => '',
            'emailBody' => '',
            'emailHeaders' => ''
        ];

        $data['emailTo'] = $this->parseEmail($this->handler['settings']['emailTo'], $seaMd, $repMd);
        $data['emailFrom'] = $this->parseEmail($this->handler['settings']['emailFrom'], $seaMd, $repMd);
        $data['subject'] = $this->mime_header_encode(str_replace($seaMd, $repMd, $this->handler['settings']['emailSubject']));
        $data['emailBody'] = str_replace($seaMd, $repMd, $this->handler['text1']);

        if (empty($data['emailTo']))
        {
            throw new NotFoundException('Not found');
        }

        if (emailMethod == 'SMTP')
        {
            $data['emailFrom'] = $this->IDN->encode(trim(SMTPLogin));
        }

        if ($data['emailFrom'] !== '' && strpos($data['emailTo'], $data['emailFrom']) !== false && $this->ord2->customer()->email())
        {
            $data['replyTo'] = $this->IDN->encode(trim($this->ord2->customer()->email()));
        }

        $files = [];
        if (!empty($this->handler['settings']['idFileSend']))
        {
            $files = $this->ord2->files();
        }
        $data = $this->parse($data, $files);

        try
        {
            if (emailMethod == 'php')
            {
                $status = @mail($data['emailTo'], $data['subject'], $data['emailBody'], $data['emailHeaders']);
                if (!$status) {
                    if (!empty(error_get_last()['message']))
                    {
                        throw new \Exception(html_entity_decode(error_get_last()['message']));
                    }
                    throw new \Exception(error2);
                }
            }
            else if (emailMethod == 'SMTP')
            {
                $settings = [
                    'SMTPSecurity' => SMTPSecurity,
                    'SMTPServer' => SMTPServer,
                    'SMTPPort' => SMTPPort,
                    'SMTPLogin' => SMTPLogin,
                    'SMTPPassword' => SMTPPassword
                ];

                $this->smtpMail($data['emailTo'], $data['emailFrom'], $data['subject'], $data['emailBody'], $data['emailHeaders'], $settings);
            }

            $this->ord2->updateOrderInfo('ha'.$this->handler['id'], [
                'baseName' => 'E-mail ('.trans('event.'.$this->ord2->event()).')',
                'value' => escape($data['emailTo']).' - Ok',
                'handler' => $this->handler['id']
            ]);
        }
        catch (\Throwable $e)
        {
            $this->ord2->updateOrderInfo('ha'.$this->handler['id'], [
                'baseName' => 'E-mail ('.trans('event.'.$this->ord2->event()).')',
                'value' => escape($data['emailTo'].' - '.$e->getMessage()),
                'handler' => $this->handler['id'],
                'error' => 1
            ]);

            if (!empty($files))
            {
                $this->ord2->setErrorSendFiles();
            }
        }

    }

    private function parseEmail($email, $seaMd, $repMd)
    {
        $email = trim(str_replace($seaMd, $repMd, $email));

        $emails = explode(',', $email);
		foreach($emails as $key=>$value)
        {
            $value = trim($value);
			$emails[$key] = $this->IDN->encode($value);
            if (empty($value)) unset($emails[$key]);
		}

        return implode(",", $emails);

    }

	private function parse($data, $atach)
    {
	    mb_internal_encoding("UTF-8");
        $data['emailHeaders'] .= "Date: ".date('r')."\r\n".
	                             "Mime-Version: 1.0\r\n".
                                 "X-Mailer: cP\r\n";

        if (!empty($data['emailFrom'])) {
            $data['emailHeaders'] .= "From: ".$data['emailFrom']."\r\n";
        }
        if (!empty($data['replyTo'])) {
            $data['emailHeaders'] .= "Reply-To: ".$data['replyTo']."\r\n";
        }
	    if (!empty($atach))
        {
            $data['emailHeaders'] .= "Content-Type: multipart/mixed;".
                                     " boundary=\"----------23DD16F3ECBCBEF\"\r\n";

            $data['emailBody'] = "------------23DD16F3ECBCBEF\r\n".
                                 "Content-Type: text/html; charset=UTF-8\r\n".
                                 "Content-Transfer-Encoding: 8bit\r\n\r\n".
                                 $data['emailBody'].
                                 "\r\n\r\n";

            $count = count($atach);
            for($i = 0; $i < $count; $i++)
            {
                $data['emailBody'] .= "------------23DD16F3ECBCBEF\r\n".
                                      "Content-Type: application/octet-stream;".
                                      " name=\"".basename($atach[$i], ".txt")."\"\r\n".
                                      "Content-Transfer-Encoding:base64\r\n".
                                      "Content-Disposition: attachment;".
                                      " filename=\"".basename($atach[$i], ".txt")."\"\r\n".
                                      "Content-ID: ".basename($atach[$i], ".txt")."\r\n\r\n";

                $content = file_get_contents(FILES_S.'orders/'.$this->ord2->id().'/'.$atach[$i]);
                if ($content) {
                    $content=chunk_split(base64_encode($content));
                }
                $data['emailBody'] .= $content."\r\n\r\n";
            }

	        $data['emailBody'] .= "\r\n------------23DD16F3ECBCBEF--";
	    }
        else
        {
            $data['emailHeaders'] .= "Content-Type: text/html; charset=UTF-8\r\n".
                                    "Content-Transfer-Encoding: 8bit\r\n";
	    }

        return $data;
	}

	private function mime_header_encode($value) {
	    $maxLength=75;
	    // RFC 2045:
	    // value needs encoding if contains non-ASCII chars or is longer than 78 chars
	    if (!preg_match('#[^\x20-\x7E]#', $value)) {
	        $token_regexp = '#([^\x21,\x23-\x27,\x2A,\x2B,\x2D'
	            . ',\x2E,\x30-\x39,\x41-\x5A,\x5E-\x7E])#';
	        if (!preg_match($token_regexp, $value)) {
	            // token
	            if (strlen($value) + 3 <= $maxLength) {
	                return $value;
	            }
	        } else {
	            // quoted-string
	            $quoted = addcslashes($value, '\\"');
	            if (strlen($quoted) + 5 <= $maxLength) {
	                return $quoted;
	            }
	        }
	    }
	    if (function_exists('mb_encode_mimeheader')) {
	        return mb_encode_mimeheader($value, 'UTF-8', 'Q');
	    } else {
	        return $value;
	    }
	}

	private function smtpMail($emailTo, $emailFrom, $subject, $emailBody, $emailHeaders, $settings)
    {
        try
        {
            $this->socket = fsockopen(($settings['SMTPSecurity'] == 'ssl' ? 'ssl://' : '').$settings['SMTPServer'], $settings['SMTPPort'], $errno, $this->smpt_response, 30);
            if (!$this->socket) {
                throw new \Exception($this->smpt_response);
            }
            $this->get();

            $code = $this->put('EHLO '.$settings['SMTPServer']."\r\n");
            if ($code != 250) {
                throw new \Exception($this->smpt_response);
            }

            if ($settings['SMTPSecurity'] == 'tls') {
                $code = $this->put("STARTTLS\r\n");
                if ($code != 250) {
                    throw new \Exception($this->smpt_response);
                }
                stream_socket_enable_crypto($this->socket, TRUE, STREAM_CRYPTO_METHOD_TLS_CLIENT);
                $code = $this->put('EHLO '.$settings['SMTPServer']."\r\n");
                if ($code != 250) {
                    throw new \Exception($this->smpt_response);
                }
            }
            $code = $this->put("AUTH LOGIN\r\n");
            if ($code != 334) {
                throw new \Exception($this->smpt_response);
            }
            $code = $this->put(base64_encode($settings['SMTPLogin'])."\r\n");
            if ($code != 334) {
                throw new \Exception($this->smpt_response);
            }
            $code = $this->put(base64_encode($settings['SMTPPassword'])."\r\n");
            if ($code != 235) {
                throw new \Exception($this->smpt_response);
            }

            $code = $this->put('MAIL FROM: <'.$emailFrom.">\r\n");
            if ($code != 250 AND $code != 251) {
                throw new \Exception($this->smpt_response);
            }

            $emails = explode(',', $emailTo);
			foreach($emails as $email) {
                $code = $this->put('RCPT TO: <'.$email.">\r\n");
                if ($code != 250 AND $code != 251) {
                    throw new \Exception($this->smpt_response);
                }
			}

            $code = $this->put("DATA\r\n");
            if ($code != 354) {
                throw new \Exception($this->smpt_response);
            }
            $code = $this->put("To: ".$emailTo."\r\nSubject: ".$subject."\r\n".
                            $emailHeaders."\r\n".
                            $emailBody."\r\n.\r\n");
            if ($code != 250) {
                throw new \Exception($this->smpt_response);
            }
        }
        catch (\Exception $e)
        {
            if ($this->socket) {
                $this->put("QUIT\r\n");
                fclose($this->socket);
            }
            throw new \Exception($settings['SMTPServer'].': '.$e->getMessage());
        }

        $this->put("QUIT\r\n");
        fclose($this->socket);

    }

    private function get()
    {
        $this->smpt_response = '';
        while ($str = fgets($this->socket, 1024)) {
            $this->smpt_response .= $str;
            if (substr($str, 3, 1) === ' ') break;
        }

        return substr($this->smpt_response,0,3);
    }

    private function put($data)
    {
        fputs($this->socket, $data);
        return $this->get();
    }

}