<?php
namespace Shop\Usecase\Handler;

use Shop\Entity\Order;
use Shop\Repository\SqlOrderRepository;
use Shop\Exception\NotFoundException;
use main;

class PaymentUsecase
{
    private $ord2;
    private $handler;
    private $orderRep;

    public function __construct(Order $ord2, $handler = [])
    {
        $this->ord2 = $ord2;
        $this->handler = $handler;
        $this->orderRep = new SqlOrderRepository();
    }

    public function run()
    {
        if (!$this->ord2->payment()->id() && !empty($this->handler['paymentId']))
        {
            $payment = $this->orderRep->findPaymentById($this->handler['paymentId'], $this->ord2->payment()->event());
            $this->ord2->setPayment($payment);
        }

        if (empty($this->ord2->price()))
        {
            main::rule('require_core', 0, array(price, ''));
            throw new NotFoundException('Not found');
        }

        if ($this->ord2->payment()->status() == 'off')
        {
            $this->ord2->updateOrderInfo('payStatus', [
                'baseName' => $this->ord2->payment()->name(),
                'value' => 'Off',
                'error' => 1
            ]);

            main::addError($this->ord2->payment()->name().' off');

            throw new NotFoundException('Payment off');
        }

        $this->ord2->updateOrderInfo('payStatus', [
            'baseName' => $this->ord2->payment()->name(),
            'value' => paymentWaitPaid
        ]);

        try
        {
            $link = main::mapper('payment'.$this->ord2->payment()->system().'_settings')->getPaymentUrl($this->ord2);
            if ($link)
            {
                $this->ord2->setRedirect($link);
            }
        }
        catch (\Throwable $e)
        {
            main::addError('Payment error');
            $this->ord2->updateOrderInfo('paymentLink', [
                'baseName' => 'Payment',
                'value' => escape($e->getMessage()),
                'error' => 1
            ]);
        }
    }

}