<?php
namespace Shop\Usecase\Handler;

use Shop\Entity\Order;
use Shop\Exception\ErrorMessageException;

class RecaptchaUsecase
{
    private $ord2;
    private $handler;

    public function __construct(Order $ord2, $handler = [])
    {
        $this->ord2 = $ord2;
        $this->handler = $handler;
    }

    public function run()
    {
        if (empty($this->handler['settings']['recaptchaSKey']))
        {
            throw new ErrorMessageException("Empty recaptcha secret key");
        }

        $vars = $this->ord2->vars();
        $seaMd = array_keys($vars);
        $repMd = array_values($vars);

        $text = str_replace($seaMd, $repMd, $this->handler['text1']);

		if (!$this->ord2->recaptchaToken())
        {
            throw new ErrorMessageException($text);
        }

        try
        {
            $url = 'https://www.google.com/recaptcha/api/siteverify';
            $params = array(
                'secret' => $this->handler['settings']['recaptchaSKey'],
                'response' => $this->ord2->recaptchaToken(),
                'remoteip' => $this->ord2->customer()->ip()
            );

            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
            curl_setopt($ch, CURLOPT_HEADER, 0);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);

            $response = curl_exec($ch);
            curl_close($ch);
        }
        catch (\Throwable $e)
        {
            $this->ord2->updateOrderInfo('ha'.$this->handler['id'], [
                'baseName' => 'Recaptcha',
                'value' => escape($e->getMessage()),
                'error' => 1
            ]);
        }

		if(!empty($response))
        {
			$response = json_decode($response);
			if (!$response->success || (float) $response->score < (float) $this->handler['settings']['recaptchaScore'])
            {
                throw new ErrorMessageException($text);
			}
		}

    }


}