<?php
namespace Shop\Usecase\Handler;

use Shop\Entity\Order;
use Shop\Exception\NotFoundException;

class TelegramUsecase
{
    private $ord2;
    private $handler;

    public function __construct(Order $ord2, $handler = [])
    {
        $this->ord2 = $ord2;
        $this->handler = $handler;
    }

    public function run()
    {
        $vars = $this->ord2->vars();
        $seaMd = array_keys($vars);
        $repMd = array_values($vars);
        $this->handler['text1'] = str_replace($seaMd, $repMd, $this->handler['text1']);

        $files = [];
        if (!empty($this->handler['settings']['idFileSend']))
        {
            $files = $this->ord2->files();
        }

        $chatIds = explode(',', $this->handler['settings']['emailTo']);
        foreach($chatIds as $i => $id)
        {
            $id = trim($id);
            $chatIds[$i] = $id;
            if (empty($id))
            {
                unset($chatIds[$i]);
            }

            if (!empty($this->handler['settings']['ids']) && isset($this->handler['settings']['ids'][$id]))
            {
                $chatIds[$i] = $this->handler['settings']['ids'][$id];
            }
        }

        $message = '';
        $chats = [];
        foreach($chatIds as $id)
        {
            try
            {
                $chats[] = $this->telergamMessage($id, $this->handler['text1'], $files);
            }
            catch (\Throwable $e)
            {
                $message = $e->getMessage();
            }
        }

        if (!empty($chats))
        {
            $this->ord2->updateOrderInfo('ha'.$this->handler['id'], [
                'baseName' => 'Telegram ('.trans('event.'.$this->ord2->event()).')',
                'value' => escape(implode(',', $chats)).' - Ok',
                'handler' => $this->handler['id']
            ]);
        }
        else
        {
            $this->ord2->updateOrderInfo('ha'.$this->handler['id'], [
                'baseName' => 'Telegram ('.trans('event.'.$this->ord2->event()).')',
                'value' => escape($this->handler['settings']['emailTo'].' - '.$message),
                'handler' => $this->handler['id'],
                'error' => 1
            ]);
            if (!empty($files))
            {
                $this->ord2->setErrorSendFiles();
            }
        }
    }

    private function telergamMessage($chatId, $text, $files)
    {
        $params = [
            'chat_id'=>$chatId,
            'parse_mode' => 'html',
            'disable_web_page_preview' => 'True',
            'text' => strip_tags(html_entity_decode(preg_replace('/(\<br(\s*)?\/?\>|\<\/div\>|\<\/p\>)/ui', "\n", $text), ENT_QUOTES, 'UTF-8'), '<b><i><a><code><pre><em><strong>')
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.telegram.org/bot'.telegramToken.'/sendMessage');
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, FALSE);
        curl_setopt($ch, CURLOPT_HEADER, FALSE);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        $response = curl_exec($ch);

        $error = curl_errno($ch);
        if ($error) {
            throw new \Exception(curl_error($ch));
        } else if ($response) {
            $response = json_decode($response, true);
            if (empty($response['ok'])) {
                throw new \Exception($response['description']);
            }
        } else {
            throw new \Exception('Error');
        }

        foreach($files as $file)
        {
            $file = FILES_S.'orders/'.$this->ord2->id().'/'.$file;
            $this->telergamDocument($ch, $chatId, $file);
        }

        curl_close($ch);

        if (!empty($response['result']['chat']['username'])) {
            return '@'.$response['result']['chat']['username'];
        }

        return $chatId;
    }

    private function telergamDocument($ch, $chatId, $file)
    {
        $document = new \CURLFile(realpath($file));
        $document->setPostFilename(basename($file, ".txt"));

        $params = ['chat_id' => $chatId,
                   'document' => $document];

        curl_setopt($ch, CURLOPT_URL, 'https://api.telegram.org/bot'.telegramToken.'/sendDocument');
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type:multipart/form-data']);
        $response = curl_exec($ch);

        $error = curl_errno($ch);
        if ($error) {
            throw new \Exception(curl_error($ch));
        } else if ($response) {
            $response = json_decode($response, true);
            if (empty($response['ok'])) {
                throw new \Exception($response['description']);
            }
        } else {
            throw new \Exception('File transer error');
        }
    }

}