<?php
namespace Shop\Usecase;

use Shop\Entity\Order;
use Shop\Repository\SqlOrderRepository;
use Shop\Exception\NotFoundException;

class ProductCodesUsecase
{
    private $ord2;
    private $orderRep;

    public function __construct(Order $ord2)
    {
        $this->ord2 = $ord2;
        $this->orderRep = new SqlOrderRepository();
    }

    public function run()
    {
        if (!$this->ord2->productIsNeedCodes() || $this->ord2->productCodes())
        {
            return;
        }

        $quantity = floor($this->ord2->productQuantity());
        if (!$quantity)
        {
            $quantity = 1;
        }

        try
        {
            $codes = $this->orderRep->getCodes($this->ord2->productId(), $quantity);

            $this->ord2->updateOrderInfo('codes', [
                'baseName' => KeysAndCodes,
                'value' => escape(implode("\n", $codes))
            ]);

            $this->ord2->removeOrderInfo('codesError');
        }
        catch (NotFoundException $e)
        {
            $this->ord2->updateOrderInfo('codesError', [
                'baseName' => KeysAndCodes,
                'value' => escape(NoAvailable),
                'error' => 1
            ]);
        }
        catch (\Throwable $e)
        {
            $this->ord2->updateOrderInfo('codesError', [
                'baseName' => KeysAndCodes,
                'value' => escape($e->getMessage()),
                'error' => 1
            ]);
        }

        $this->orderRep->unlock($this->ord2->productId());
    }

}