
class cpContextMenu extends cpMenu {

    lastObj = null;
    isSetHide = 1;

    zindex;

    constructor(editor) {

        super(editor);

        this.render();
        this.menu = document.getElementById('cpContextMenu');

        document.getElementById("delCont").addEventListener("click", this.deleteMenu);
        document.getElementById("copyCont").addEventListener("click", this.copy);
        document.getElementById("pasteCont").addEventListener("click", this.paste);
        this.menu.querySelector(".uppTool").addEventListener("click", this.uppCanvas);
        this.menu.querySelector(".downTool").addEventListener("click", this.downCanvas);
        this.menu.querySelector(".hideCont").addEventListener("click", this.hideCommand);

    }

    copy = (event) => {

        this.editor.executeCommand(
            new cpCopyCommand(this.editor, event.pageY, event.pageX)
        );

    }

    paste = () => {

        this.editor.executeCommand(
            new cpPasteCommand(this.editor,
                               this.editor.nodes[0].querySelector('.droppable'),
                               this.editor.mouseClick.left,
                               this.editor.mouseClick.top)
        );

    }

    show(event = null, obj = null) {

        if (obj) {
            this.lastObj = obj;
        } else if (this.lastObj) {
            obj = this.lastObj;
        }

        this.isShow = this.lastObj.id;

        removeClass(this.menu, "hide2");

        if (event) {
            this.menu.style.top = (event.pageY + 3) + 'px';
            this.menu.style.left = (event.pageX + 3) + 'px';
        }

        if (this.editor.nodes.length > 1) {
            this.settingsMultyElements();
        } else {
            this.settingsOneElements();
        }
        this.editor.mobileMenu.hide();

        if (this.zindex) this.zindex.save();

        document.getElementById("zPlus").addEventListener("mousedown", this.zindexCounter);
        document.getElementById("zMinus").addEventListener("mousedown", this.zindexCounter);

    }

    hide() {

        super.hide();
        if (this.zindex) {
            this.zindex.save();
            document.getElementById("zPlus").removeEventListener("mousedown", this.zindexCounter);
            document.getElementById("zMinus").removeEventListener("mousedown", this.zindexCounter);
        }

    }

    zindexCounter = (event) => {
        if (this.zindex) this.zindex.save();
        this.zindex = new cpZindex(this.editor, event);
    }

    deleteMenu = (event) => {

        this.editor.changeState(cpDeleteState).init(event);

    }

    uppCanvas = (event) => {
        this.editor.move.top(event);
    }

    downCanvas = (event) => {
        this.editor.move.bottom(event);
    }

    hideCommand = () => {

        this.editor.executeCommand(
            new cpHideCommand(this.editor, this.editor.nodes[0], this.isSetHide)
        );

    }

    render() {

        document.body.insertAdjacentHTML("beforeend", `

            <div id="cpContextMenu" class="hide2">
                <div id="uppdown" class="cmItem">
                  <div class="uppTool" title="${contextConst.MoveT}"></div>
                  <div class="downTool" title="${contextConst.MoveT}"></div>
                </div>
                <a href="#" class="acmItem aedit" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="settingCont" class="cmItem">
                        <span class="cmsp">
                            ${contextConst.main_settings}
                            <i></i>
                        </span>
                    </div>
                </a>
                <a href="#" class="acmItem alist" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="menuListCont" class="cmItem">
                        <span class="cmsp">${contextConst.menuList_core}</span>
                    </div>
                </a>
                <a href="#" class="acmItem alist" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="listCont" class="cmItem">
                        <span class="cmsp">${contextConst.selectList}</span>
                    </div>
                </a>
                <a href="#" class="acmItem afields" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="listFields" class="cmItem">
                        <span class="cmsp">${contextConst.formFields}</span>
                    </div>
                </a>
                <a href="./?service=handler_core" class="acmItem ahandlers" onclick="top.location=this.getAttribute('href');return false;">
                    <div class="cmItem handlersCont">
                        <span class="cmsp">${contextConst.handlers}</span>
                    </div>
                </a>
                <a href="#" class="acmItem alink" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="linkCont" class="cmItem">
                        <span class="cmsp">${contextConst.addMenu}</span>
                    </div>
                </a>
                <a href="#" class="acmItem amenu" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="menuCont" class="cmItem">
                        <span class="cmsp">${contextConst.addMenuDropDown}</span>
                    </div>
                </a>
                <a href="#" class="acmItem aimage" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="imageCont" class="cmItem">
                        <span class="cmsp">${contextConst.addImage}</span>
                    </div>
                </a>
                <a href="#" class="acmItem atext" onclick="top.location=this.getAttribute('href');return false;">
                    <div id="textCont" class="cmItem">
                        <span class="cmsp">${contextConst.addText}</span>
                    </div>
                </a>
                <div class="cmItem hideCont">
                    <span class="cmsp">
                        <span id="bhide1">${contextConst.bhide}</span>
                        <span id="viewItem1">${contextConst.viewItem}</span>
                    </span>
                </div>
                <div id="zCont" class="cmItem">
                    <span class="cmsp">
                        <span id="zMinus"> - </span>&nbsp;
                        <b>1</b>&nbsp;
                        <span id="zPlus"> + </span>
                    </span>
                </div>
                <div id="pasteCont" class="cmItem">
                    <span class="cmsp">
                        ${contextConst.pasteTool}
                        <i></i>
                    </span>
                </div>
                <div id="copyCont" class="cmItem multyContext">
                    <span class="cmsp">
                        ${contextConst.cloneBlock_core}
                        <i></i>
                    </span>
                </div>
                <div id="delCont" class="cmItem multyContext">
                    <span class="cmsp">
                        ${contextConst.bDel}
                        <i></i>
                    </span>
                </div>
            </div>

        `);

    }

    settingsMultyElements() {

        let $mainMenu = $(this.menu);

        $mainMenu.find('.cmItem:not(#delCont):not(#settingCont)').addClass('hide2');
        $mainMenu.find('i').text(' ('+this.editor.nodes.length+')');

        if (hasClass(this.lastObj, "cblockM")) $mainMenu.find('#copyCont').addClass('hide2');
        else $mainMenu.find('#copyCont').removeClass('hide2');

        let ids1 = [];
        for(let i = 0, length = this.editor.nodes.length; i < length; i++) {
            ids1.push(this.editor.nodes[i].id.substr(1));
        }

        $mainMenu.find('.aedit').attr("href", "./edit.php?id="+ this.editor.pageId +"&service=adaptive_core&action=edit&blockId=0&swidth="+top.cPointWidth+top.allPointWidth+"&ids="+ids1.join(',')+"&startStr="+(hasClass(this.lastObj, "virtuals") ? '9' : '1'));

    }

    settingsOneElements() {
        let $mainMenu = $(this.menu);

        this.isSetHide = 1;
        $mainMenu.find('i').text('');
        $mainMenu.find('.cmItem').removeClass("hide2");
        this.selId = this.lastObj.id.substr(1);

        $mainMenu.find('.aimage').attr("href", this.setHref('edit', 'image_gallery', true));
        $mainMenu.find('.atext').attr("href", this.setHref('edit', 'textC_core', true));
        $mainMenu.find('.alink').attr("href", this.setHref('edit', 'link_core', true));
        $mainMenu.find('.amenu').attr("href", this.setHref('edit', 'menu_core', true));
        $mainMenu.find('.aedit').attr("href", this.setHref('edit'));
        $mainMenu.find('.alist').attr("href", this.setHref('lists'));

        if (hasClass(this.lastObj, "gallContext"))
        {
            $mainMenu.find('#uppdown,#linkCont,#menuCont,#menuListCont,#pasteCont,#listFields,.handlersCont').addClass('hide2');
        }
        else if (hasClass(this.lastObj, "gallContext0"))
        {
            $mainMenu.find('#uppdown,#linkCont,#menuCont,#menuListCont,#pasteCont,#listFields,.handlersCont').addClass('hide2');
        }
        else if (hasClass(this.lastObj, "menuListContext"))
        {
            $mainMenu.find('#uppdown,#textCont,#imageCont,#listCont,#pasteCont,#listFields,.handlersCont').addClass('hide2');
        }
        else if (hasClass(this.lastObj, "menuContext"))
        {
            $mainMenu.find('#uppdown,#textCont,#imageCont,#menuCont,#listCont,#pasteCont,#listFields,.handlersCont').addClass('hide2');
        }
        else if (hasClass(this.lastObj, "virtuals"))
        {
            $mainMenu.find('#imageCont,#textCont,#linkCont,#menuCont,#menuListCont,#listFields,.handlersCont'+(copyIndexes == '' ? ',#pasteCont':'')).addClass('hide2');

            let pgselectedDr = document.getElementById('pgc_'+this.selId),
                cps = getComputedStyle(pgselectedDr);

            if (cps.display === "none" || cps.opacity === "0.44") this.isSetHide = 0;
        }
        else if (hasClass(this.lastObj, "drrr0"))
        {
            $mainMenu.find('#uppdown,#imageCont,#textCont,#linkCont,#menuCont,#menuListCont,#pasteCont,#listFields,.handlersCont').addClass('hide2');
        }
        else if (hasClass(this.lastObj, "formContext"))
        {
            let form = this.lastObj.querySelector('[form]'),
                formId = 0;

            if (form)
            {
                formId = form.getAttribute("form").substr(4);
            }

            $mainMenu.find('.ahandlers').attr("href", './?service=handler_core&formId=' + formId);
            $mainMenu.find('.afields').attr("href", "./edit.php?id="+ this.editor.pageId +"&service=form_forms&action=listsFields&formId="+formId+"&swidth="+top.cPointWidth+top.allPointWidth);

            $mainMenu.find('#uppdown,#imageCont,#textCont,#linkCont,#menuCont,#menuListCont,#listCont,#pasteCont').addClass('hide2');
        }
        else
        {
            $mainMenu.find('#uppdown,#imageCont,#textCont,#linkCont,#menuCont,#menuListCont,#listCont,#pasteCont,#listFields,.handlersCont').addClass('hide2');
        }

        if (!hasClass(this.lastObj, "virtuals")) {
            let cps = getComputedStyle(this.lastObj);
            if (cps.display === "none" || cps.opacity === "0.44") this.isSetHide = 0;
        }

        if (this.isSetHide === 0) {$mainMenu.find('.hideCont').addClass('shohide');}
        else {$mainMenu.find('.hideCont').removeClass('shohide');}

        if (getComputedStyle(this.lastObj).position === "absolute" ||
            hasClass(this.lastObj, "virtuals")) {

            $mainMenu.find('b').text(parseInt(this.lastObj.getAttribute("data-zindex")));

        } else {

            $mainMenu.find('#zCont').addClass('hide2');

        }

        if (hasClass(this.lastObj, "cblockM")) $mainMenu.find('#copyCont').addClass('hide2');

    }

}