class cpAbsoluteMove extends cpElementsMove
{
    dropStart;
    constructor(editor, initObj, nodes)
    {
        super(editor, initObj, nodes);
        this.position = new cpPosition(editor, initObj, 'absolute');
    }

    top(event, inc = -1)
    {
        if (this.autoSaveTime == 0) this.correct(1);

        for(let i=0; i < this.length; i++)
        {
            this.nodes[i].style.top = (parseFloat(this.nodes[i].style.top) + inc) + 'px';
        }

        this.refresh(event);
    }

    left(event, inc = -1)
    {
        if (this.autoSaveTime == 0) this.correct(1);

        for(let i=0; i < this.length; i++) {
            this.nodes[i].style.left = (parseFloat(this.nodes[i].style.left) + inc) + 'px';
        }

        this.refresh(event);
    }

    right(event)
    {
        this.left(event, 1);
    }

    bottom(event)
    {
        this.top(event, 1);
    }

    dragStart = (event, ui) =>
    {
        if (this.editor.state.deepClickTimerId)
        {
            clearTimeout(this.editor.state.deepClickTimerId);
            this.editor.state.deepClickTimerId = 0;
        }

        this.editor.contextMenu.hide();
        this.editor.mobileMenu.hide();
        this.editor.stopClick();

        this.search(event);
        this.position.set(ui.position.left, ui.position.top);

        for(let i = 0; i < this.length; i++)
        {
          addClass(this.nodes[i], "pointerNone");
        }

        this.correct(0.5);
    }

    drag = (event, ui) =>
    {
        for(let i = 0; i < this.length; i++)
        {
            setCss(this.nodes[i], {
                top: (parseFloat(this.nodes[i].style.top) + (ui.position.top - this.position.top))+"px",
                left: (parseFloat(this.nodes[i].style.left) + (ui.position.left - this.position.left))+"px"
            });
        }

        this.position.set(ui.position.left, ui.position.top);
        this.search(event);
    }

    dragEnd = (event) =>
    {
        let parentNode = this.initObj.parentNode;
        this.editor.clickCoords(event, parentNode);

        let correctLeft = this.editor.mouseClick.left,
            correctTop = this.editor.mouseClick.top;

        this.search(event);

        let parentDrop;
        if (this.parentId && this.parentId != this.startParentId)
        {
            parentDrop = document.getElementById('c_'+this.parentId);
            if (!parentDrop)
            {
                parentDrop = document.getElementById('b'+this.parentId);
            }
            this.editor.clickCoords(event, parentDrop);
        }

        let dropII = document.querySelector('.dropII');
        if (dropII)
        {
            dropII.classList.remove("dropII");
        }

        for(let i = 0; i < this.length; i++)
        {
            let css = getComputedStyle(this.nodes[i]);
            setCss(this.nodes[i],
            {
                top: (parseFloat(css.top) + this.editor.mouseClick.top - correctTop) + "px",
                left: (parseFloat(css.left) + this.editor.mouseClick.left - correctLeft) + "px",
                opacity: ""
            });

            removeClass(this.nodes[i], 'pointerNone');

            if (parentDrop)
            {
                parentDrop.appendChild(this.nodes[i]);
            }
        }

        if (parentDrop)
        {
            this.editor.layers.expandMap(parentNode.closest(".blockM,.dCnt2"));
            this.editor.layers.expandMap(this.initObj.parentNode.closest(".blockM,.dCnt2"));
            this.editor.layers.scroll(this.initObj);
        }

        this.save();

        if (this.dropStart)
        {
            this.dropStart.classList.remove("dropStart");
        }
        this.startParentId = 0;
    }

    save()
    {
        this.editor.executeCommand(
            new cpAbsoluteMoveCommand(this.editor, this.nodes, this.parentId)
        );

        return super.save();
    }

    correct(opacity)
    {
        for(let i = 0; i < this.length; i++)
        {
            let css = getComputedStyle(this.nodes[i]);
            setCss(this.nodes[i],
            {
                top: (Math.round($(this.nodes[i]).position().top) + parseFloat(css.marginTop)) + "px",
                left: (Math.round($(this.nodes[i]).position().left) + parseFloat(css.marginLeft)) + "px",
                right: "auto",
                bottom: "auto",
                transform: "none",
                margin: 0,
                opacity: opacity
            });
        }
    }

    search = (event) =>
    {
        if (this.initObj.classList.contains("cblockM") || !event.altKey)
        {
            if (!this.startParentId)
            {
                let parent = this.initObj.parentNode.closest(".blockM,.virtuals");
                parent.classList.add("dropII");
                this.startParentId = this.parentId = parent.id.substr(1);
            }
            return;
        }

        if (!this.dropStart)
        {
            this.dropStart = this.initObj.closest(".virtuals");
            this.dropStart.classList.add("dropStart");
        }

        let el = document.elementFromPoint(event.clientX, event.clientY);
        if (!el)
        {
            return;
        }

        el = el.closest(".drrr0,.virtuals");
        if (el)
        {
            this.hoverDrop(el);
        }
    }
}
