class cpRelativeMove extends cpElementsMove
{
    dropStart;
    constructor(editor, initObj, nodes)
    {
        super(editor, initObj, nodes);
        this.position = new cpPosition(editor, initObj, 'relative');
    }

    top(event)
    {
        this.position.previous();
        this.refresh(event);
    }

    bottom(event)
    {
        this.position.next();
        this.refresh(event);
    }

    left(event)
    {
        this.top(event);
    }

    right(event)
    {
        this.bottom(event);
    }


    dragStart = (event, ui) =>
    {
        if (this.editor.state.deepClickTimerId)
        {
            clearTimeout(this.editor.state.deepClickTimerId);
            this.editor.state.deepClickTimerId = 0;
        }

        this.editor.stopClick();
        this.editor.contextMenu.hide();
        this.editor.mobileMenu.hide();
        this.position.line.hide();

        this.search(event);
        this.position.set(ui.position.left, ui.position.top);

        for(let i = 0, length = this.nodes.length; i < length; i++)
        {
            addClass(this.nodes[i], "pointerNone");

            let css = getComputedStyle(this.nodes[i]);
            setCss(this.nodes[i], {
                opacity: 0.5,
                zIndex: 5500 + this.nodes[i].dataset.zindex,
                top: css.top,
                left: css.left,
                right: "auto",
                bottom: "auto"
            });

        }

        this.position.show();
    }

    drag = (event, ui) =>
    {
        for(let i = 0, length = this.nodes.length; i < length; i++)
        {
            setCss(this.nodes[i], {top: (parseFloat(this.nodes[i].style.top) + (ui.position.top - this.position.top))+"px",
                                   left: (parseFloat(this.nodes[i].style.left) + (ui.position.left - this.position.left))+"px"});
        }

        this.position.set(ui.position.left, ui.position.top);
        this.search(event);
    }

    dragEnd = (event, ui) =>
    {
        this.search(event);

        let parentDrop = document.getElementById('c_'+this.parentId);
        if (!parentDrop)
        {
            parentDrop = document.getElementById('b'+this.parentId);
        }

        this.editor.clickCoords(event, parentDrop);

        this.position.set(ui.position.left, ui.position.top);

        let dropII = document.querySelector('.dropII');
        if (dropII)
        {
            dropII.classList.remove("dropII");
        }

        for(let i = 0, length = this.nodes.length; i < length; i++)
        {
            removeClass(this.nodes[i], 'pointerNone');
            setCss(this.nodes[i], {top: "",
                                bottom: "",
                                  left: "",
                                 right: "",
                              position: "",
                               opacity: "",
                                zIndex: ""});
        }

        this.save();

        if (this.dropStart)
        {
            this.dropStart.classList.remove("dropStart");
        }
        this.startParentId = 0;
    }

    save()
    {
        if (this.position.insert ||
            this.parentId !== this.initObj.parentNode.closest(".blockM,.virtuals").id.substr(1))
        {
            this.editor.executeCommand(
                new cpRelativeMoveCommand(this.editor, this.position, this.nodes, this.parentId)
            );
        }

        return super.save();
    }

    search = (event) =>
    {
        if (this.initObj.classList.contains("cblockM") || !event.altKey)
        {
            return this.simpleSearch(event);
        }

        if (!this.dropStart)
        {
            this.dropStart = this.initObj.closest(".virtuals");
            this.dropStart.classList.add("dropStart");
        }

        let el = document.elementFromPoint(event.clientX, event.clientY);
        if (!el)
        {
            return;
        }

        el = el.closest(".blockM,.virtuals");
        if (!el)
        {
            return;
        }

        if (hasClass(el, 'virtuals'))
        {
            this.hoverDrop(el);
            this.position.line.hide();
            return;
        }

        if (getComputedStyle(el).position != "relative")
        {
            el = el.closest(".drrr0");
            if (!el || getComputedStyle(el).position != "relative")
            {
                this.position.line.hide();
                if (el) this.hoverDrop(el);
                return;
            }

        }

        var box = el.getBoundingClientRect(),
            corrOffTop = box.height <= 10 ? 0 : 10,
            corrOffLeft = box.width <= 10 ? 0 : 10,
            clientYY = box.top + box.height - corrOffTop;

        if (hasClass(el, "drrr0"))
        {
           var clientXX = box.left + box.width - corrOffLeft;

           if (event.clientX >= box.left + corrOffLeft &&
               event.clientX <= clientXX &&
               event.clientY >= box.top + corrOffTop &&
               event.clientY <= clientYY)
            {
                this.position.line.hide();
                this.hoverDrop(el);
                return;
            }

        }
        else
        {
            var clientXX = box.left + box.width / 2;
        }


        this.position.insert = "first";
        if (event.clientX >= clientXX || (clientYY && event.clientY >= clientYY))
        {
            this.position.insert = "last";
        }

        this.position.node = el;
        this.position.line.show();

        var parent = this.position.node.parentNode.closest(".drrr0,.virtuals");
        this.hoverDrop(parent);
    }

    simpleSearch = (event) =>
    {
        if (!this.startParentId)
        {
            let parent = this.initObj.parentNode.closest(".blockM,.virtuals");
            parent.classList.add("dropII");
            this.startParentId = this.parentId = parent.id.substr(1);
        }

        let el = document.elementFromPoint(event.clientX, event.clientY);
        if (!el)
        {
            return;
        }

        el = el.closest(".cblockM,.blockM");
        if (!el)
        {
            return;
        }

        let parent = el.parentNode.closest(".blockM,.virtuals");
        while (parent = el.parentNode.closest(".blockM,.virtuals"))
        {
            if (this.parentId === parent.id.substr(1) && getComputedStyle(el).position === "relative")
            {
                break;
            }
            el = parent;
        }

        if (!parent) return;

        let box = el.getBoundingClientRect(),
            corrOffTop = box.height <= 10 ? 0 : 10,
            clientXX = box.left + box.width / 2,
            clientYY = box.top + box.height - corrOffTop;

        this.position.insert = "first";
        if (event.clientX >= clientXX || (clientYY && event.clientY >= clientYY))
        {
            this.position.insert = "last";
        }

        this.position.node = el;
        this.position.line.show();
    }
}
