
class cpEditState extends cpState
{
    constructor(editor)
    {
        super(editor);
        editor.toolbar.setState('cpEditState');
        document.addEventListener("mousedown", this.mousedown, {capture: true, passive: true});

    }

    pointerover = (event) =>
    {
        event.stopPropagation();
        let obj = event.currentTarget;
        if(0 in this.editor.nodes && this.findParentAndChildSelected(obj))
        {
            return;
        }

        resizableOn(obj);
    }

    findParentAndChildSelected(obj)
    {
        let list = obj.querySelectorAll(".blockM.ui-selected,.cblockM.ui-selected");
        for(let i=0, length = list.length; i < length; i++)
        {
            return true;
        }

        while (obj = obj.parentNode.closest(".blockM.ui-selected,.cblockM.ui-selected,.dCnt2.ui-selected"))
        {
            return true;
        }
        return false;
    }

    mousedown = (event) =>
    {
        if (event.button == 2 || this.isTouch == true || event.target.classList.contains('ui-resizable-handle'))
        {
            return;
        }
        let target = event.target.closest('.blockM,.cblockM');
        if (target)
        {
            cpElementsMove.dragInit(this.editor, target);
        }
        else
        {
            canvasSelectable(event.target);
        }

    }

    touchstart(event)
    {
        event.stopPropagation();
        this.isTouch = true;
        let obj = event.currentTarget;
        this.deepClickTimerId = setTimeout(() => this.deepСlick(obj, event), 2000);
        window.addEventListener("touchend", this.stopClick);
    }

    click(event)
    {
        if (this.deepClickTimerId)
        {
            clearTimeout(this.deepClickTimerId);
            this.deepClickTimerId = 0;
        }

        event.stopPropagation();
        event.preventDefault();

        this.editor.clickCoords(event);

        let obj = event.currentTarget;

        if (this.isTouch == true)
        {
            if (this.editor.mobileMenu.isShow == obj.id)
            {
                this.editor.executeCommand(
                    new cpClearCommand(this.editor)
                );
            }
            else
            {
                if (obj.closest('.blockM,.cblockM'))
                {
                    cpElementsMove.dragInit(this.editor, obj);
                }
                else
                {
                    this.editor.executeCommand(
                        new cpSelectOneCommand(this.editor, obj)
                    );
                }
                this.editor.mobileMenu.show(event);
            }
            this.isTouch = false;
            return;
        }

        this.editor.executeCommand(
            new cpSelectOneCommand(this.editor, obj)
        );
    }

    dblclick(event)
    {
        $(".ui-draggable").draggable("destroy");

        event.stopPropagation();
        event.preventDefault();

        let obj = event.currentTarget;

        if (!event.altKey && hasClass(obj, 'cpedit') && hasClass(obj, 'ui-selected') && obj.querySelector("[contenteditable]"))
        {
            resizableOn(obj);
            this.editor.changeState(cpEditTextState).init(event, obj);
            return;
        }

        this.editor.deepSelect(event);
    }

    stopClick = () =>
    {
        window.removeEventListener("touchend", this.stopClick);
        if (this.deepClickTimerId)
        {
            clearTimeout(this.deepClickTimerId);
            this.deepClickTimerId = 0;
        }
    }

    deepСlick(obj, event)
    {
        this.deepClickTimerId = 0;

        if (hasClass(obj, 'cpedit') && obj.querySelector("[contenteditable]"))
        {
            this.editor.stopClickEvent(event);
            event.stopPropagation();
            $(".ui-draggable").draggable("destroy");
            this.editor.changeState(cpEditTextState).init(event, obj);
        }
    }

    finish()
    {
        document.removeEventListener("mousedown", this.mousedown, {capture: true, passive: true});
        $(".ui-draggable").draggable("destroy");
        selectableDestroy(this.editor);

        this.editor.save();
    }

    keydown(event)
    {
        switch(event.which)
        {
            case 37:
                this.editor.move.left(event);
                break;
            case 38:
                this.editor.move.top(event);
                break;
            case 39:
                this.editor.move.right(event);
                break;
            case 40:
                this.editor.move.bottom(event);
                break;
            default:
                super.keydown(event);
        }
    }
}
